#include <chessboard.h>

#include <algo.h>
#include <fstream>

const char *  ChessBoard::stdBoard[] =
{ "RNBQKBNR",
  "PPPPPPPP",
  "        ",
  "        ",
  "        ",
  "        ",
  "pppppppp",
  "rnbqkbnr" };



ChessBoard::ChessBoard(const char * data[])
{
  for( int y=0; y < Chess::YSIZE; y++ )
    {
      const char *s = data[y];
      for( int x=0; x < Chess::XSIZE; x++ )
	at(ChessCoord(x,y)) = ChessPiece(*s++);
    }
}


ChessUndoBoardMove ChessBoard::move(ChessMove move)
{
  assert(move.getStart() != move.getEnd() );
  assert(at(move.getStart()).piece != ChessPiece::EMPTY); 
  assert(!(at(move.getStart()).piece == ChessPiece::PAWN  &&
	 (move.getEnd().isUp() || move.getEnd().isDown()) &&
	 move.getPiece() == ChessPiece::EMPTY));
  ChessPiece
    &startP=at(move.getStart()),
    &endP=at(move.getEnd());
  ChessUndoBoardMove undoM(move.getStart(), startP.piece,
		       move.getEnd(), endP.piece,
		       flags);
  if( startP.piece == ChessPiece::PAWN &&
      ( move.getEnd().isUp() || move.getEnd().isDown() ))
      endP.piece= move.getPiece();
  else
    endP.piece = startP.piece;
  endP.color = startP.color;
  startP.piece = ChessPiece::EMPTY;
  return undoM;
}

ostream& operator<<(ostream& os, ChessBoard& board)
{
  for( int i=0; i < 1+2*Chess::YSIZE; i++ )
    {
      if( i%2 == 0 )
	{
	  for( int space=0; space < 5; space++)
	    os << " ";
	  for( int minus=0; minus <= 8*4; minus++ )
	    os << "-";
	}
      else
	{
	  int row=i/2;
	  os << "  " << 8 - row << "  ";
	  for(int minus=0; minus <= Chess::XSIZE*4; minus++)
	    if( minus % 4 == 0 )
	      os << "|";
	    else
	      if( (minus-2) % 4 == 0 )
		os << 
		  board[ChessCoord((minus-2)/4,Chess::YSIZE-1-row)].toChar();
	      else
		os << " ";
	}
      os << endl;
    }
  os << "    ";
  for( char c = 'A'; c < 'A'+Chess::XSIZE; c++ )
    os << "   " << c;
  os << endl;
  return os;
}

ChessBoard::ChessBoard(const ChessBoard& cb)
  : flags(cb.flags)
{
  copy(cb.board, cb.board+Chess::XSIZE*Chess::YSIZE, board);
}

ChessBoard::ChessBoard(const string data, format_t format)
{
  switch( format )
    {
    case ICS:
      parseICS(data);
      break;
    case FEN:
      cerr << "FEN Board not implemented" << endl;
      break;
    case  PGN:
      cerr << "PGN Board not implemented" << endl;
      break;
    }
}


void ChessBoard::parseICS(const string& s)
{
  cout << "[" << s.substr(0,4) << "]" << endl;
  assert( s.substr(0,5) == "<12> " );
  for( int y=0; y<8; y++ )
    for( int x=0; x<8; x++ )
      at(x,7-y)=ChessPiece(s[5+y*9+x]);
  int index=5+8*9;
  assert(s[index]=='B' || s[index]=='W' );
  flags.setColToMove(s[index]=='W'?ChessPiece::WHITE:ChessPiece::BLACK);
  index+=2;
  char c=s[index];
  if( c == '-' ) /* -1 */
    {
      index++;
      flags.setEnpassat(-1);
    }
  else
    {
      assert( c >= '0' && c <= '7' );
      flags.setEnpassat( c-'0' );
    }
  index+=2;
  for( int i=ChessPiece::WHITE; i <= ChessPiece::BLACK; i++ )
    {
      assert( s[index] == '1' || s[index] == '0' );
      assert( s[index+2] == '1' || s[index+2] == '0' );
      flags.setCastle( ChessPiece::color_t(i), 
		       s[index] == '1' ?  ChessBoardFlags::SHORT : 0 +
		       s[index+2] == '1' ? ChessBoardFlags::LONG : 0 );
      index+=4;
    }
#ifdef CHESSBOARD_USE_NAMES
  int space=0;
  while( space < 2 )
    {
      assert( index < s.size() );
      if( s[index++] == ' ' )
	space++;
    }
  int start=index, count=0;
  for( int i=0; i< 2; i++ )
    {
      int start=index, count=0;
      while( s[index++] != ' ' ) 
	{
	  assert(index < s.size() );
	  count++;
	}
      playerName[i]=s.substr(start,count-1);
    }
#endif
}

void ChessBoard::writePovrayInc(char * filename){
  #ifdef __CHESS_NEWSIZE__
  #warning the povray template file won t work
  #endif
  ofstream pov(filename);
  pov << "/* " << filename << endl;
  pov << " automatic created by ChessLib" << endl << endl;
  pov << "Input= " << endl << *this << endl << "*/" << endl;
  for( int color = ChessPiece::WHITE; color <= ChessPiece::BLACK; color++ )
    {
      pov << "#declare ";
      if( color == ChessPiece::WHITE ) 
	pov << "White"; 
      else
	pov << "Black";
      pov << "Side = \n  union {" << endl;
      insertPovPieces(pov, ChessPiece::color_t(color));
      pov << "}"  << endl << endl << endl;
    }
}

void ChessBoard::insertPovPieces(ostream& os, ChessPiece::color_t col)
{
  assert( col != ChessPiece::NOCOLOR );
  char prefix= col == ChessPiece::WHITE ? 'W' : 'B';
  for( int i=0; i < Chess::XSIZE*Chess::YSIZE; i++ )
    if( at(i).getPiece() != ChessPiece::EMPTY &&
	at(i).getColor() == col )
      {
	os << "    object { " << prefix << at(i).toName() 
	   << " translate < " 
	   << -28+8*ChessCoord(i).getX() << ", 0, "
	   << -28+8*ChessCoord(i).getY() << " > } " << endl;
	  
      }
  
}  


string ChessBoard::fenString()
{
  string fen="";
  for( int y=Chess::YSIZE-1; y >=0; y-- )
    {
      char empty=0;
      for( int x=0; x < Chess::XSIZE; x++ )
	if( at(x, y).empty() )
	  empty++;
	else
	  {
	    if( empty != 0 )
	      {
		fen+='0'+ empty;
		empty=0;
	      }
	    fen+=at(x, y).toChar();
	  }
      if( empty != 0 )
	fen += '0' +empty;
      if( y )
	fen += '/';
    }
  return fen;
}

string ChessBoard::gicsSetBoardString()
{
  return "exset " + fenString() + " " +
    flags.getColToMoveChar();
}

	      

      


	
	
  
