/* file: ChessLib/chessboard.h
   author: Christoph Bauer
   date: 22.1.00
   license: GPL

   ChangeLog:
     o 22.1.00 created

   Description:
     Board ist im Grunde eine Klasse um ein field[64] 
       o mit Informationen wie
           oo wer ist am Zug
	   oo en passat
	   oo Rochaden
       o mit ein paar Umwandlungsfunktionen.
*/

#ifndef __CHESSBOARD_H__
#define __CHESSBOARD_H__

#include <iostream>
#include <string>

#include <chessconfig.h>

#include <chesspiece.h>
#include <chesscoord.h>
#include <chessundoboardmove.h>
#include <chessmove.h>


class ChessComprBoard;

class ChessBoard
{
  friend class ChessUndoBoardMove;
 public:
  typedef long long hash_t;
  typedef ChessPiece * iterator;
  typedef const ChessPiece * const_iterator;
  
  enum format_t { 
    ICS, FEN, PGN
  }; /* formats */

  static const char * stdBoard[];

#ifdef CHESSBOARD_USE_NAMES
  string playerName[2];
#endif
  
 private:
  ChessPiece board[Chess::XSIZE*Chess::YSIZE];
  ChessBoardFlags flags;
  
  void parseICS(const string &s);
  void insertPovPieces(ostream& os, ChessPiece::color_t col);
  
 public:
  ChessBoard(const char * data[] = stdBoard);
  ChessBoard(const string data, format_t format = ICS); 
  ChessBoard(const ChessBoard& cb);
  ChessBoard(const ChessComprBoard& cb);

  hash_t getHashKey();
  
  string icsString();
  string fenString();
  string arrayString();
  string texString();

  string gicsSetBoardString();
  string ficsSetBoardString();

#ifdef CHESSBOARD_USE_NAMES
  string getPlayerName(ChessPiece::color_t col)
    { return playerName[col]; }
  void setPlayerName(ChessPiece::color_t col, string name)
    { playerName[col]=name; }
#endif

  void writePovrayInc(char * filename = "myboard.inc");

  iterator begin()
    { return board; }
  const_iterator begin() const
    { return board; }
  iterator end()
    { return board+Chess::XSIZE*Chess::YSIZE; }
  const_iterator end() const
    { return board+Chess::XSIZE*Chess::YSIZE; }

  iterator line_begin(int line)
  { return board+line*Chess::XSIZE; }
  iterator line_end(int line)
  { return board+(line+1)*Chess::XSIZE; }
  
  
  void setCastle(ChessPiece::color_t color, 
		 unsigned int enable = ChessBoardFlags::BOTH)
  { flags.setCastle(color,enable); }
  unsigned int getCastle(ChessPiece::color_t color)
  { return flags.getCastle(color); }

  void setWCastle( unsigned int enable = ChessBoardFlags::BOTH)
  { flags.setWCastle(enable); }
  void setBCastle( unsigned int enable = ChessBoardFlags::BOTH)
  { flags.setBCastle(enable); }

  unsigned int getWCastle()
  { return flags.getWCastle(); }
  unsigned int getBCastle()
  { return flags.getBCastle(); }

  void setColToMove(ChessPiece::color_t col)
  { flags.setColToMove(col); }
  ChessPiece::color_t getColToMove()
  { return flags.getColToMove(); }

  void setEnpassat(int column)
  { flags.setEnpassat(column); }
  int getEnpassat()
  { return flags.getEnpassat(); }
  
  bool isLegal();

  ChessUndoBoardMove move(ChessMove move);
  ChessUndoBoardMove move(char * s)
    { return move(ChessMove(s)); }
  
  void undo(const ChessUndoBoardMove& m)
    { m.undo(*this); }

  ChessPiece& at(ChessCoord c)
    { return board[c.getIndex()]; }
  ChessPiece& at(ChessCoord::coord_t x, ChessCoord::coord_t y)
    { return at(ChessCoord(x,y)); }

  const ChessPiece& at(ChessCoord c) const
    { return board[c.getIndex()]; }
  const ChessPiece& at(ChessCoord::coord_t x, ChessCoord::coord_t y) const
    { return at(ChessCoord(x,y)); }

  
  
  ChessPiece& operator[](ChessCoord c)
    { return at(c); }
  ChessPiece& operator[](char * s)
    { return at(ChessCoord(s)); }
  ChessPiece& operator[](int i)
    { return at(ChessCoord(i)); }
  ChessPiece& operator()(ChessCoord::coord_t x, ChessCoord::coord_t y)
    { return at(ChessCoord(x, y)); }
};

extern
ostream& operator<<(ostream& os, ChessBoard& cb);

#endif
  
  
  



   

   
   
