/* file: ChessLib/chesscoords.h
   author: Christoph Bauer
   date: 21.1.00
   license: GPL
   
   ChangeLog:
     o 21.1.00 created
   
   Description:
     Abstraktion von (x,y) Koordinaten.
  
     Die Klasse ChessCoord erlaubt es verschiedene Formate in ein
     Standardformat umzuwandeln. 

     Hier kann getestet werden, welch Implemtierung effizienter ist:
       o einen index (0-63) speichern
       o zwei Indizes x,y speichern
*/

#ifndef __CHESSCOORD_H__
#define __CHESSCOORD_H__

#include <assert.h>
#include <iostream.h>
#include <string>
#include <assert.h>

#include <chesscommon.h>

class ChessInitChessTabs;

class ChessCoord
{
  friend class ChessInitChessTabs;
  typedef int coord_t;
  
  coord_t i;
  static bool boardTable[(Chess::XSIZE+4)*(Chess::YSIZE*4)];
  static int countKTab[Chess::XSIZE*Chess::YSIZE];
  static coord_t knightTab[Chess::XSIZE*Chess::YSIZE*8];

 public:
  ChessCoord() {}
  ChessCoord(coord_t x, coord_t y)
    : i(x+y*Chess::XSIZE) 
    { assert(isInBoard(x,y)); }
  ChessCoord(const ChessCoord& c)
    : i(c.i)
    {}
  ChessCoord(const coord_t index)
    : i(index)
    {}
  /* For strings like "e4": */
  ChessCoord(string s)
    {
      assert(isLegal(s));
      i=(s[0]-'a')+(s[1]-'1')*Chess::XSIZE;
    }

  static bool isInBoard(coord_t x, coord_t y)
    { return x>=0 && y>=0 && x<Chess::XSIZE && y<Chess::YSIZE; }

  /* leave the Board is faster than isInBoard, but only defined for
     x in -2..CHESSXSIZE+2 and y in -2..CHESSYSIZE+2  and
     Normal Chesspieces can check with this function wether they leave the 
     Board 
  */
  template<class T>
    static bool leavesTheBoard(T x, T y)
    { assert(x >= -2 && y >= -2 && x < Chess::XSIZE+2 && y < Chess::YSIZE+2);
      return boardTable[x+y*Chess::XSIZE]; }

  static bool isLegal(string s)
    { return (s.length() == 2) && 
	isInBoard(s[0]-'a',s[1]-'1'); }
  
#ifdef __CHESS_NEWSIZE_
  coord_t getX() const
    { return i % Chess::XSIZE; }
  coord_t getY() const
    { return i / Chess::XSIZE; }
#else
  coord_t getX() const
    { return i & 07; }
  coord_t getY() const
    { return (i & 070) >> 3; }
#endif

  coord_t getIndex() const
    { return i; }

  bool isLeft() const
    { return getX() == 0; }
  bool isRight() const
    { return getX() == Chess::XSIZE-1; }
  bool isUp() const
    { return getY() == 0;  }
  bool isDown() const
    { return getY() == Chess::YSIZE-1;}
  
  ChessCoord& left()
    { assert(!isLeft()); i--; return *this; }
  ChessCoord& right()
    { assert(!isRight()); i++; return *this; }
  ChessCoord& up()
    { assert(!isUp()); i-=Chess::XSIZE; return *this; }
  ChessCoord& down()
    { assert(!isDown()); i+=Chess::XSIZE; return *this; }

  int countKnightMoves() const
    { return countKTab[i]; }
  ChessCoord knightMove(int n)
    { assert(n > 0 && n < countKnightMoves()); return ChessCoord(knightTab[i+n*Chess::XSIZE*Chess::YSIZE]); }
  
  bool operator==(ChessCoord c) const
  { return c.i == i; }
  bool operator!=(ChessCoord c) const
  { return c.i != i; }
};

extern
ostream& operator<<(ostream& os, ChessCoord c);



#endif 


  
  

  
